# -*- coding: utf-8 -*-

# Version follows PEP 440 for PyPI publishing
# https://www.python.org/dev/peps/pep-0440/
__version__ = VERSION = '0.1.0rc1'

import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())
